@@ -505,10 +505,13 @@ static bool ocl_makeFrameFromDepth(const UMat depth, OutputArrayOfArrays points,
         return false;
 
     // depth truncation can be used in some scenes
+    UMat depthThreshold;
     if(truncateThreshold > 0.f)
-        threshold(depth, depth, truncateThreshold*depthFactor, 0.0, THRESH_TOZERO_INV);
+        threshold(smooth, depthThreshold, truncateThreshold * depthFactor, 0.0, THRESH_TOZERO_INV);
+    else
+        depthThreshold = smooth;
 
-    UMat scaled = smooth;
+    UMat scaled = depthThreshold;
     Size sz = smooth.size();
     points.create(levels, 1, POINT_TYPE);
     normals.create(levels, 1, POINT_TYPE);
@@ -624,13 +627,16 @@ void makeFrameFromDepth(InputArray _depth,
     bilateralFilter(depth, smooth, kernelSize, sigmaDepth*depthFactor, sigmaSpatial);
 
     // depth truncation can be used in some scenes
+    Depth depthThreshold;
     if(truncateThreshold > 0.f)
-        threshold(depth, depth, truncateThreshold, 0.0, THRESH_TOZERO_INV);
+        threshold(smooth, depthThreshold, truncateThreshold * depthFactor, 0.0, THRESH_TOZERO_INV);
+    else
+        depthThreshold = smooth;
 
     // we don't need depth pyramid outside this method
     // if we do, the code is to be refactored
 
-    Depth scaled = smooth;
+    Depth scaled = depthThreshold;
     Size sz = smooth.size();
     pyrPoints.create(levels, 1, POINT_TYPE);
     pyrNormals.create(levels, 1, POINT_TYPE);
