@@ -8,35 +8,38 @@
 #include "trackerCSRTUtils.hpp"
 #include "trackerCSRTScaleEstimation.hpp"
 
-namespace cv
-{
+namespace cv {
+inline namespace tracking {
+namespace impl {
+
 /**
 * \brief Implementation of TrackerModel for CSRT algorithm
 */
-class TrackerCSRTModel : public TrackerModel
+class TrackerCSRTModel CV_FINAL : public TrackerModel
 {
 public:
-    TrackerCSRTModel(TrackerCSRT::Params /*params*/){}
+    TrackerCSRTModel(){}
     ~TrackerCSRTModel(){}
 protected:
     void modelEstimationImpl(const std::vector<Mat>& /*responses*/) CV_OVERRIDE {}
     void modelUpdateImpl() CV_OVERRIDE {}
 };
 
-
-class TrackerCSRTImpl : public TrackerCSRT
+class TrackerCSRTImpl CV_FINAL : public TrackerCSRT
 {
 public:
-    TrackerCSRTImpl(const TrackerCSRT::Params &parameters = TrackerCSRT::Params());
-    void read(const FileNode& fn) CV_OVERRIDE;
-    void write(FileStorage& fs) const CV_OVERRIDE;
+    TrackerCSRTImpl(const Params &parameters = Params());
+
+    Params params;
+
+    Ptr<TrackerCSRTModel> model;
+
+    // Tracker API
+    virtual void init(InputArray image, const Rect& boundingBox) CV_OVERRIDE;
+    virtual bool update(InputArray image, Rect& boundingBox) CV_OVERRIDE;
+    virtual void setInitialMask(InputArray mask) CV_OVERRIDE;
 
 protected:
-    TrackerCSRT::Params params;
-
-    bool initImpl(const Mat& image, const Rect2d& boundingBox) CV_OVERRIDE;
-    virtual void setInitialMask(InputArray mask) CV_OVERRIDE;
-    bool updateImpl(const Mat& image, Rect2d& boundingBox) CV_OVERRIDE;
     void update_csr_filter(const Mat &image, const Mat &my_mask);
     void update_histograms(const Mat &image, const Rect &region);
     void extract_histograms(const Mat &image, cv::Rect region, Histogram &hf, Histogram &hb);
@@ -49,7 +52,6 @@ protected:
     Point2f estimate_new_position(const Mat &image);
     std::vector<Mat> get_features(const Mat &patch, const Size2i &feature_size);
 
-private:
     bool check_mask_area(const Mat &mat, const double obj_area);
     float current_scale_factor;
     Mat window;
@@ -75,28 +77,10 @@ private:
     int cell_size;
 };
 
-Ptr<TrackerCSRT> TrackerCSRT::create(const TrackerCSRT::Params &parameters)
-{
-    return Ptr<TrackerCSRTImpl>(new TrackerCSRTImpl(parameters));
-}
-Ptr<TrackerCSRT> TrackerCSRT::create()
-{
-    return Ptr<TrackerCSRTImpl>(new TrackerCSRTImpl());
-}
 TrackerCSRTImpl::TrackerCSRTImpl(const TrackerCSRT::Params &parameters) :
     params(parameters)
 {
-    isInit = false;
-}
-
-void TrackerCSRTImpl::read(const cv::FileNode& fn)
-{
-    params.read(fn);
-}
-
-void TrackerCSRTImpl::write(cv::FileStorage& fs) const
-{
-    params.write(fs);
+    // nothing
 }
 
 void TrackerCSRTImpl::setInitialMask(InputArray mask)
@@ -463,13 +447,13 @@ Point2f TrackerCSRTImpl::estimate_new_position(const Mat &image)
 // *********************************************************************
 // *                        Update API function                        *
 // *********************************************************************
-bool TrackerCSRTImpl::updateImpl(const Mat& image_, Rect2d& boundingBox)
+bool TrackerCSRTImpl::update(InputArray image_, Rect& boundingBox)
 {
     Mat image;
     if(image_.channels() == 1)    //treat gray image as color image
         cvtColor(image_, image, COLOR_GRAY2BGR);
     else
-        image = image_;
+        image = image_.getMat();
 
     object_center = estimate_new_position(image);
     if (object_center.x < 0 && object_center.y < 0)
@@ -506,13 +490,13 @@ bool TrackerCSRTImpl::updateImpl(const Mat& image_, Rect2d& boundingBox)
 // *********************************************************************
 // *                        Init API function                          *
 // *********************************************************************
-bool TrackerCSRTImpl::initImpl(const Mat& image_, const Rect2d& boundingBox)
+void TrackerCSRTImpl::init(InputArray image_, const Rect& boundingBox)
 {
     Mat image;
     if(image_.channels() == 1)    //treat gray image as color image
         cvtColor(image_, image, COLOR_GRAY2BGR);
     else
-        image = image_;
+        image = image_.getMat();
 
     current_scale_factor = 1.0;
     image_size = image.size();
@@ -545,8 +529,7 @@ bool TrackerCSRTImpl::initImpl(const Mat& image_, const Rect2d& boundingBox)
     } else if(params.window_function.compare("kaiser") == 0) {
         window = get_kaiser_win(Size(yf.cols,yf.rows), params.kaiser_alpha);
     } else {
-        std::cout << "Not a valid window function" << std::endl;
-        return false;
+        CV_Error(Error::StsBadArg, "Not a valid window function");
     }
 
     Size2i scaled_obj_size = Size2i(cvFloor(original_target_size.width * rescale_ratio / cell_size),
@@ -616,11 +599,11 @@ bool TrackerCSRTImpl::initImpl(const Mat& image_, const Rect2d& boundingBox)
     dsst = DSST(image, bounding_box, template_size, params.number_of_scales, params.scale_step,
             params.scale_model_max_area, params.scale_sigma_factor, params.scale_lr);
 
-    model=Ptr<TrackerCSRTModel>(new TrackerCSRTModel(params));
-    isInit = true;
-    return true;
+    model=makePtr<TrackerCSRTModel>();
 }
 
+}  // namespace impl
+
 TrackerCSRT::Params::Params()
 {
     use_channel_weights = true;
@@ -652,94 +635,21 @@ TrackerCSRT::Params::Params()
     psr_threshold = 0.035f;
 }
 
-void TrackerCSRT::Params::read(const FileNode& fn)
+TrackerCSRT::TrackerCSRT()
 {
-    *this = TrackerCSRT::Params();
-    if(!fn["padding"].empty())
-        fn["padding"] >> padding;
-    if(!fn["template_size"].empty())
-        fn["template_size"] >> template_size;
-    if(!fn["gsl_sigma"].empty())
-        fn["gsl_sigma"] >> gsl_sigma;
-    if(!fn["hog_orientations"].empty())
-        fn["hog_orientations"] >> hog_orientations;
-    if(!fn["num_hog_channels_used"].empty())
-        fn["num_hog_channels_used"] >> num_hog_channels_used;
-    if(!fn["hog_clip"].empty())
-        fn["hog_clip"] >> hog_clip;
-    if(!fn["use_hog"].empty())
-        fn["use_hog"] >> use_hog;
-    if(!fn["use_color_names"].empty())
-        fn["use_color_names"] >> use_color_names;
-    if(!fn["use_gray"].empty())
-        fn["use_gray"] >> use_gray;
-    if(!fn["use_rgb"].empty())
-        fn["use_rgb"] >> use_rgb;
-    if(!fn["window_function"].empty())
-        fn["window_function"] >> window_function;
-    if(!fn["kaiser_alpha"].empty())
-        fn["kaiser_alpha"] >> kaiser_alpha;
-    if(!fn["cheb_attenuation"].empty())
-        fn["cheb_attenuation"] >> cheb_attenuation;
-    if(!fn["filter_lr"].empty())
-        fn["filter_lr"] >> filter_lr;
-    if(!fn["admm_iterations"].empty())
-        fn["admm_iterations"] >> admm_iterations;
-    if(!fn["number_of_scales"].empty())
-        fn["number_of_scales"] >> number_of_scales;
-    if(!fn["scale_sigma_factor"].empty())
-        fn["scale_sigma_factor"] >> scale_sigma_factor;
-    if(!fn["scale_model_max_area"].empty())
-        fn["scale_model_max_area"] >> scale_model_max_area;
-    if(!fn["scale_lr"].empty())
-        fn["scale_lr"] >> scale_lr;
-    if(!fn["scale_step"].empty())
-        fn["scale_step"] >> scale_step;
-    if(!fn["use_channel_weights"].empty())
-        fn["use_channel_weights"] >> use_channel_weights;
-    if(!fn["weights_lr"].empty())
-        fn["weights_lr"] >> weights_lr;
-    if(!fn["use_segmentation"].empty())
-        fn["use_segmentation"] >> use_segmentation;
-    if(!fn["histogram_bins"].empty())
-        fn["histogram_bins"] >> histogram_bins;
-    if(!fn["background_ratio"].empty())
-        fn["background_ratio"] >> background_ratio;
-    if(!fn["histogram_lr"].empty())
-        fn["histogram_lr"] >> histogram_lr;
-    if(!fn["psr_threshold"].empty())
-        fn["psr_threshold"] >> psr_threshold;
-    CV_Assert(number_of_scales % 2 == 1);
-    CV_Assert(use_gray || use_color_names || use_hog || use_rgb);
+    // nothing
 }
-void TrackerCSRT::Params::write(FileStorage& fs) const
+
+TrackerCSRT::~TrackerCSRT()
 {
-    fs << "padding" << padding;
-    fs << "template_size" << template_size;
-    fs << "gsl_sigma" << gsl_sigma;
-    fs << "hog_orientations" << hog_orientations;
-    fs << "num_hog_channels_used" << num_hog_channels_used;
-    fs << "hog_clip" << hog_clip;
-    fs << "use_hog" << use_hog;
-    fs << "use_color_names" << use_color_names;
-    fs << "use_gray" << use_gray;
-    fs << "use_rgb" << use_rgb;
-    fs << "window_function" << window_function;
-    fs << "kaiser_alpha" << kaiser_alpha;
-    fs << "cheb_attenuation" << cheb_attenuation;
-    fs << "filter_lr" << filter_lr;
-    fs << "admm_iterations" << admm_iterations;
-    fs << "number_of_scales" << number_of_scales;
-    fs << "scale_sigma_factor" << scale_sigma_factor;
-    fs << "scale_model_max_area" << scale_model_max_area;
-    fs << "scale_lr" << scale_lr;
-    fs << "scale_step" << scale_step;
-    fs << "use_channel_weights" << use_channel_weights;
-    fs << "weights_lr" << weights_lr;
-    fs << "use_segmentation" << use_segmentation;
-    fs << "histogram_bins" << histogram_bins;
-    fs << "background_ratio" << background_ratio;
-    fs << "histogram_lr" << histogram_lr;
-    fs << "psr_threshold" << psr_threshold;
+    // nothing
 }
-} /* namespace cv */
+
+Ptr<TrackerCSRT> TrackerCSRT::create(const TrackerCSRT::Params &parameters)
+{
+    return makePtr<TrackerCSRTImpl>(parameters);
+}
+
+}}  // namespace
+
+#include "legacy/trackerCSRT.legacy.hpp"
